// UPDATE DETAILS OF A FIRST MANAGED ITEMS ENTRY OF DEFINED CHANGE ORDER
// Before using this script, create a new Forge app at forge.autodesk.com (this service is for free)
// Please provide clientId and clientSecret of your Forge app in file settings.js before 
// Author: Sven Dickmans, Autodesk

/* --------------------------------------------------------------------------------------------------------
    Change Log
    - 2022-02-16: Integration in TS F3M Extensions Package
    - 2021-12-10: Initial version
   -------------------------------------------------------------------------------------------------------- */


//Options
let wsId        = '84';         // Equals to workspace Change Orders in default tenant
let dmsId       = '14245';      // ID of Change Order to be updated


const axios     = require('axios');
const f3m       = require('../node_modules_adsk/f3m.js');
const utils     = require('../node_modules_adsk/utils.js');
const settings  = require('../settings.js');


utils.printStart([ ['Workspace', wsId], ['DMSID', dmsId] ]);

f3m.login().then(function() {
    getManagedItems(function(data) {
        updateManagedItem(data, function() {
            utils.printEnd();
        });
    });
});


function getManagedItems(callback) {
    
    let url = 'https://' + settings.tenant + '.autodeskplm360.net/api/v3/workspaces/' + wsId + '/items/' + dmsId + '/views/11';
    
    axios.get(url).then(function (response) {
        
        console.log('    Found ' + response.data.affectedItems.length + ' affected items');
        console.log();
        console.log();
        console.log('    Available Properties');
        
        utils.printLine();

        for (var key in response.data.affectedItems[0]) {
            console.log('    ' + key);
        }
        
        console.log();
        console.log();
        console.log('    List of affected items');
        
        utils.printLine();
        
        for(affectedItem of response.data.affectedItems) {
            
            console.log('    ' + affectedItem.item.title);
            
        }
        
        callback(response.data);
        
    }).catch(function (error) {
        console.log(error.message);
    });
    
}

function updateManagedItem(data, callback) {

    let affectedItem = data.affectedItems[0];
    
    let url = 'https://' + settings.tenant + '.autodeskplm360.net' + affectedItem.__self__;
    
    let params = {
        'linkedFields' : [{
            'value' : 'HQ',
            '__self__' : '/api/v3/workspaces/84/views/11/fields/LOCATION'
        }],
    }

    axios.put(url, params).then(function (response) {
        callback();
    }).catch(function (error) {
        console.log(error.message);
    });
    
}